#!/bin/bash

SCRIPT_VERSION=5

# initialize a ton of system variables
CONFIGSVCS_ROOT=`grep "CONFIGSERVICES_ROOT=" $NVA_CONF | cut -d= -f2`
NVACONF_DIR=`grep "NVACONF=" $NVA_CONF | cut -d= -f2`
NVABIN_DIR=`grep "NVABIN=" $NVA_CONF | cut -d= -f2`

DBNAME="qradar"
DBUSER=`grep "DBUSER=" $NVA_CONF | cut -d= -f2`
DBPASS=`grep "DBPASS=" $NVA_CONF | cut -d= -f2`

EXISTING_XML_LOC="${NVACONF_DIR}/templates"
XML_FILE="qidmap-import.xml"
SQL_FILE="prepare-qidmap-import.sql"
IMPORT_SCRIPT="apply_qidmap.pl"

declare -a DEVICE_LIST
DEVICE_COUNT=0
declare -a DEVICE_FIELDS
FIELD_COUNT=0
declare -a PROTOS
PROTO_COUNT=0

# takes three args: string, separator and a flag
# it will clear the target array before use to 
# ensure that it is complete and consistent with string
function parseArray()
{
	ARRAY_STRING=$1
	SEP=$2
	INDEX=0
	FLAG=$3
	
	# check for terminating separator to avoid infinite loop
	if [ "X${ARRAY_STRING##*$SEP}" != "X" ]; then
		ARRAY_STRING="${ARRAY_STRING}$SEP"
	fi
	# now, chomp and add to array
	while true
	do
		if [ $FLAG -eq 0 ]; then
			DEVICE_LIST[$INDEX]=${ARRAY_STRING%%$SEP*}
		elif [ $FLAG -eq 1 ]; then
			DEVICE_FIELDS[$INDEX]=${ARRAY_STRING%%$SEP*}
		else
			PROTOS[$INDEX]=${ARRAY_STRING%%$SEP*}
		fi
		ARRAY_STRING=${ARRAY_STRING#*$SEP}
		((INDEX=$INDEX+1))		
		if [ "X${ARRAY_STRING}" == "X" ]; then
			break;
		fi
	done
	if [ $FLAG -eq 0 ]; then
		DEVICE_COUNT=$INDEX
	elif [ $FLAG -eq 1 ]; then
		FIELD_COUNT=$INDEX
	else
		PROTO_COUNT=$INDEX
	fi
}

function insertMultipleDevices()
{
	DEVICE_ARRAY="$1"
	DEVICE_VERSION="$2"

	unset DEVICE_LIST
	DEVICE_COUNT=0
	parseArray "${DEVICE_ARRAY}" "|" "0"
	for (( DEVICE_INDEX=0; $DEVICE_COUNT>$DEVICE_INDEX; DEVICE_INDEX+=1 ))
	do
		unset DEVICE_FIELDS
		FIELD_COUNT=0
		parseArray "${DEVICE_LIST[$DEVICE_INDEX]}" "^" "1"
		if [ "$FIELD_COUNT" != "7" ]; then
			echo "error inserting device, incorrect argument count: ${FIELD_COUNT}. Should be 7"
		fi
		unset PROTOS
		PROTO_COUNT=0
		parseArray "${DEVICE_FIELDS[6]}" "," "2"
		psql -U qradar -c "SELECT insert_device(${DEVICE_FIELDS[0]},'${DEVICE_FIELDS[1]}','${DEVICE_FIELDS[2]}',${DEVICE_FIELDS[3]},'${DEVICE_FIELDS[4]}',${DEVICE_FIELDS[5]},${PROTOS[0]},'${DEVICE_VERSION}');" >> /tmp/dbUpdate.log;
		for (( PROTO_INDEX=1; $PROTO_COUNT>$PROTO_INDEX; PROTO_INDEX+=1 )) 
		do
			psql -U qradar -c "SELECT addSensorDeviceProtocol(${DEVICE_FIELDS[0]},${PROTOS[$PROTO_INDEX]});" >> /tmp/dbUpdate.log;
		done
	done
}

function removeMultipleDevices()
{
	DEVICE_ARRAY="$1"
	DEVICE_VERSION="$2"
	unset DEVICE_LIST
	DEVICE_COUNT=0
	parseArray "${DEVICE_ARRAY}" "|" "0"
	for (( DEVICE_INDEX=0; $DEVICE_COUNT>$DEVICE_INDEX; DEVICE_INDEX+=1 ))
	do
		unset DEVICE_FIELDS
		FIELD_COUNT=0
		parseArray "${DEVICE_LIST[$DEVICE_INDEX]}" "^" "1"
		if [ "$FIELD_COUNT" != "7" ]; then
			echo "error deleting device, incorrect argument count: ${FIELD_COUNT}. Should be 7"
		fi		

		echo "Removing old instances of '${DEVICE_FIELDS[2]}'  Device";
		psql -U qradar -c "SELECT dsm_rpm_removed(${DEVICE_FIELDS[0]},'${DEVICE_VERSION}');" >> /tmp/dbUpdate.log;
	done
}