package AutoupdateDeploy;
use 5.006;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(copyProtocolJars copyUIConfig updateECvmTemplate);

#!/usr/bin/perl -w

use strict;
use File::Copy;

=pod

=head1 NAME

autoupdate-deploy_v2_util.pl

=head1 DESCRIPTION

Some utility functions to make installing sources a little easier.
Version: 1.1

=head2 Methods

=over

=item copyProtocolJars()

Copies the new/updated protocol jars to the proper location (from where the RPM installed them).

=item copyProtocolJarsWithKey($)

Copies the new/updated protocol jars to the proper location (from where the RPM installed them) based on a provided file globbing pattern

=item copyUIConfig()

Copies the new/updated protocol pages and supporting config files to the proper locations (from where the RPM installed them).

=item updateECvmTemplate($$)

Updates the EC.vm velocity template so that source metadata can be added or updated.

arg1 - Source info in the form of a hash
arg2 - Source type info in the form of a hash

=back

=cut

my %statusStack = ();
my $confDirectory = "/opt/qradar/conf/";
my $webappsDirectory = "/opt/qradar/webapps/console/WEB-INF/lib/";

sub copyProtocolJars()
{
	my $protocolJarPattern = "q1labs_semsources_protocol_*.jar";
	my $updatesDir = "./updateplugins";
	my $protocolDir = "../jars";
	my $success = 0;
	my $error = 0;

	my @JARLIST=split "\n", `cd $updatesDir; ls -1 $protocolJarPattern`;
	foreach my $protocolJar ( @JARLIST )
	{
		print "Moving protocol jar ".$updatesDir."/".$protocolJar." to ".$protocolDir."\n";

		# Copy to the webapps directory.
		$success = copy( $updatesDir."/".$protocolJar, $webappsDirectory."/" );
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to copy protocol jar $protocolJar";
			$error = $!;
		}

		# Move it to the jars directory.
		$success = move( $updatesDir."/".$protocolJar, $protocolDir."/" );
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to move protocol jar $protocolJar";
			$error = $!;
		}
	}

	return 	$error;	
}

sub copyProtocolJarsWithKey($)
{
	my $protocolJarPattern = shift;
	my $updatesDir = "./updateplugins";
	my $protocolDir = "../jars";
	my $success = 0;
	my $error = 0;

	if (!$protocolJarPattern || $protocolJarPattern eq "")
	{	
		$statusStack{"ErrorText"} = "No protocol jar pattern was passed to copyProtocolJarsWithKey($)";
		$error = 1;
	}
	else
	{
		my @JARLIST=split "\n", `cd $updatesDir; ls -1 $protocolJarPattern`;
		foreach my $protocolJar ( @JARLIST )
		{
			print "Moving protocol jar ".$updatesDir."/".$protocolJar." to ".$protocolDir."\n";

			# Copy to the webapps directory.
			$success = copy( $updatesDir."/".$protocolJar, $webappsDirectory."/" );
			if($success eq 0)
			{
				$statusStack{"ErrorText"} =  "Unable to copy protocol jar $protocolJar";
				$error = $!;
			}

			# Move it to the jars directory.
			$success = move( $updatesDir."/".$protocolJar, $protocolDir."/" );
			if($success eq 0)
			{
				$statusStack{"ErrorText"} =  "Unable to move protocol jar $protocolJar";
				$error = $!;
			}
		}
	}

	return 	$error;	
}

sub copyUIConfig()
{
	my $error = 0;
	my $success = 0;
	my $webrootDirectory = "/opt/qradar/webapps/console/sem/jsp/sensorDevice/";
	my $appconfigDirectory = $confDirectory."appconfig/";
	my $localizationDirectory = $confDirectory."localization/";
	my $updatesDir = "./updateplugins";

	print "Moving UI config files...\n";

	# JSP pages - e.g. SdeeConfig.jsp -> /opt/qradar/webapps/console/sem/jsp/sensorDevice/SdeeConfig.jsp
	my @JSPLIST=split "\n", `cd $updatesDir; ls -1 *.jsp`;

	foreach my $jsp (@JSPLIST)
	{
		print "Moving protocol config page ".$updatesDir."/".$jsp." to ".$webrootDirectory."\n";

		$success = move( $updatesDir."/".$jsp, $webrootDirectory."/" );
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to move protocol config page $jsp";
			$error = $!;
		}
	}

	# appconfig files - e.g. appconfig.sdee.properties -> /opt/qradar/conf/appconfig/sdee.properties
	my @APPCONFIGPROPLIST=split "\n", `cd $updatesDir; ls -1 appconfig.*.properties`;
	my $newFilename;
	
	foreach my $appconfig (@APPCONFIGPROPLIST)
	{
		$appconfig =~ m/appconfig\.(.*?\.properties)$/;
		$newFilename = $1;
		print "Moving protocol app config file ".$updatesDir."/".$appconfig." to ".$appconfigDirectory."$newFilename\n";

		$success = move( $updatesDir."/".$appconfig, $appconfigDirectory.$newFilename);
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to move protocol app config file $appconfig";
			$error = $!;
		}
	}

	# message mapping files - e.g. message-mapping.sdee.properties -> /opt/qradar/conf/message-mapping.sdee.properties
	my @MESSAGEMAPLIST=split "\n", `cd $updatesDir; ls -1 message-mapping.*.properties`;

	foreach my $msgmapFile (@MESSAGEMAPLIST)
	{
		print "Moving message map config file ".$updatesDir."/".$msgmapFile." to ".$confDirectory."\n";

		$success = move( $updatesDir."/".$msgmapFile, $confDirectory);
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to move message map config file $msgmapFile";
			$error = $!;
		}
	}

	# localization files - e.g. localization.sdee.properties ->/opt/qradar/conf/localization/sdee.properties
	my @LOCALIZATIONPROPLIST=split "\n", `cd $updatesDir; ls -1 localization.*.properties`;

	foreach my $localizationconfig (@LOCALIZATIONPROPLIST)
	{
		$localizationconfig =~ m/localization\.(.*?\.properties)$/;
		$newFilename = $1;
		print "Moving localization config file ".$updatesDir."/".$localizationconfig." to ".$localizationDirectory."$newFilename\n";

		$success = move( $updatesDir."/".$localizationconfig, $localizationDirectory.$newFilename);
		if($success eq 0)
		{
			$statusStack{"ErrorText"} =  "Unable to move localization config file $localizationconfig";
			$error = $!;
		}
	}

	return 	$error;	
}

sub mergeSourceInfo(\%\%\%\%)
{
	(my $sourcetypeinfo, my $sourceinfo, my $newsourcetypeinfo, my $newsourceinfo) = (\%{$_[0]}, \%{$_[1]}, \%{$_[2]}, \%{$_[3]});

	if(defined $newsourcetypeinfo->{"objectId"})
	{
		# See if it already exists; if it does replace it.
		my $newEntry = ""; 

		# Construct the tab-sep metadata
		$newEntry = $newEntry."description=".$newsourcetypeinfo->{"description"}."\tschema=".$newsourcetypeinfo->{"schema"}."\tclass=".$newsourcetypeinfo->{"class"}."\tsource=".$newsourcetypeinfo->{"source"}."\ttype=".$newsourcetypeinfo->{"type"};

		# Assign it into the hash
		$sourcetypeinfo->{$newsourcetypeinfo->{"objectId"}} = $newEntry;
	}

	if(defined $newsourceinfo->{"objectId"})
	{
		# See if it already exists; if it does replace it.
		my $newEntry = "";
		my $maxParameterIndex = keys(%$newsourceinfo);

		# Construct the tab-sep metadata
		$newEntry = $newEntry."stdout=".$newsourceinfo->{"stdout"}."\ttype=".$newsourceinfo->{"type"};

		# Process the parameters. They should be zero-indexed and must be consecutive.
		for(my $index=0; $index<$maxParameterIndex; $index++)
		{
			if(defined $newsourceinfo->{"parameter".$index})
			{
				$newsourceinfo->{"parameter".$index} =~ m/(.*?)=(.*?)$/;

				$newEntry = $newEntry."\tparameter_$1=$2";
			}
		}

		# Assign it into the hash
		$sourceinfo->{$newsourceinfo->{"objectId"}} = $newEntry;
	}
}

sub updateECvmTemplate(\%\%)
{
	(my $newsourceinfo, my $newsourcetypeinfo) = (\%{$_[0]}, \%{$_[1]});

	my $templatesDir = $confDirectory."templates/configservices/";
	my $ECTemplateFile = $templatesDir."EC.vm";
	my $startTag = "<!-- PLUGGABLE SOURCES START -->\n";
	my $endTag = "<!-- PLUGGBALE SOURCES END -->\n";
	my $isProcessing = 0;
	my $processingSource = 0;
	my $processingSourceType = 0;
	my %sources;
	my %sourcetypes;
	my $currentEntry;
	my $currentObjectId;
	my $ecvmFileOld;
	my $ecvmFileNew;

	# Open the template file and keep everything that falls outside the sources --> destinations comments.
	open $ecvmFileOld, "<", $ECTemplateFile;
	open $ecvmFileNew, ">", $ECTemplateFile."new";

	while(<$ecvmFileOld>)
	{
		# Find the start / end of our "pluggable" block
		if($isProcessing && $_ eq $endTag)
		{
			$isProcessing = 0;

			# We're done, so print out all of our stuff here.
			if($newsourceinfo || $newsourcetypeinfo)
			{
				mergeSourceInfo(%sourcetypes, %sources, %{$newsourcetypeinfo}, %{$newsourceinfo});
			}

			# Source types first
			foreach my $id (keys(%sourcetypes))
			{
				my @values = split('\t', $sourcetypes{$id});
				my %parammap;
				
				for my $vals (@values)
				{
					$vals =~ tr/\t//;
					(my $name, my $val) = split("=", $vals);

					if(defined($val))
					{
						$parammap{$name} = $val;
					}
					elsif(defined($name))
					{
						$parammap{$name} = "";
					}
				}

				print $ecvmFileNew "\t<sourceType description=\"".$parammap{"description"}."\" objectId=\"".$id."\" schema=\"".$parammap{"schema"}."\">\n";
				print $ecvmFileNew "\t\t<implement class=\"".$parammap{"class"}."\" source=\"".$parammap{"source"}."\" type=\"".$parammap{"type"}."\"/>\n";
				print $ecvmFileNew "\t</sourceType>\n";
			}

			print $ecvmFileNew "\n";

			# Now the sources themselves
			foreach my $id (keys(%sources))
			{
				my @values = split('\t', $sources{$id});
				my %parammap;

				for my $vals (@values)
				{
					(my $name, my $val) = split("=", $vals);

					if(defined($val))
					{
						$parammap{$name} = $val;
					}
					elsif(defined($name))
					{
						$parammap{$name} = "";
					}
				}

				print $ecvmFileNew "\t<source objectId=\"".$id."\" stdout=\"".$parammap{"stdout"}."\" type=\"".$parammap{"type"}."\">\n";
				while($sources{$id} =~ m/parameter_(.*?)=(.*?)(?:\t|$)/g)
				{
					print $ecvmFileNew "\t\t<parameter type=\"".$1."\">$2</parameter>\n";
				}
				print $ecvmFileNew "\t</source>\n";
			}

			# Print the closing tag last
			print $ecvmFileNew $endTag;
			next;
		}
		elsif(!$isProcessing && $_ eq $startTag)
		{
			$isProcessing = 1;

			# Print our opening tag then begin.
			print $ecvmFileNew $startTag;
			next;
		}

		# A state machine for gathering what we need and writing out the rest.
		if($isProcessing)
		{
			if($_ =~ m/^\W*<source (?:disabled="(False|True)" )?objectId="(.*?)" stdout="(.*?)" type="(.*?)">/)
			{
				# Start source section
				$processingSource = 1;
				$currentObjectId = $2;
				$currentEntry = "";

				print "Starting source section $currentObjectId...\n    ";
				if(defined($1))
				{
					print "disabled = $1, ";
					$currentEntry = $currentEntry."\tdisabled=$1";
				}
				print "stdout = $3, type = $4\n";
				$currentEntry = $currentEntry."\tstdout=$3\ttype=$4";
			}
			elsif($processingSource && $_ =~ m#^\W*</source>#)
			{
				# End source section
				print "Ending source section.\n\n";
				$processingSource = 0;
				$sources{$currentObjectId} = $currentEntry;
			}
			elsif($_ =~ m/^\W*<sourceType description="(.*?)" objectId="(.*?)" schema="(.*?)"/)
			{
				# Start source type section
				$processingSourceType = 1;
				$currentObjectId = $2;
				$currentEntry = "";

				print "Starting source type section $2...\n   ";
				print "description = $1, schema = $3\n";
				$currentEntry = $currentEntry."\tdescription=$1\tschema=$3";
			}
			elsif($processingSourceType && $_ =~ m#^\W*</sourceType>#)
			{
				# End source type section
				print "Ending source type section.\n\n";
				$processingSourceType = 0;
				$sourcetypes{$currentObjectId} = $currentEntry;
			}
			elsif($processingSource && $_ =~ m#<parameter type="(.*?)">(.*?)</parameter>$#)
			{
				# Source parameter
				print "Found source parameter $1 = $2\n";
				$currentEntry = $currentEntry."\tparameter_$1=$2";
			}
			elsif($processingSourceType && $_ =~ m#<implement class="(.*?)" source="(.*?)" type="(.*?)"/>$#)
			{
				# Source type implementation
				print "Found source type implementation info class = $1, source = $2, type = $3\n";
				$currentEntry = $currentEntry."\tclass=$1\tsource=$2\ttype=$3";
			}
		}
		else
		{
			print $ecvmFileNew $_;
		}
	}

	close $ecvmFileNew;
	close $ecvmFileOld;
	
	# Well we made it this far. Backup the old one and copy the new one into its place.
	move($ECTemplateFile, $ECTemplateFile.".".time().".bak");
	move($ECTemplateFile."new", $ECTemplateFile);
	
	return 0;
}
