#!/usr/bin/python

# Version 2

scriptVersion = "3.0"

import os
import sys
import re
from time import sleep
from optparse import OptionParser

# This is the query to select all of the known event collectors from the local database
strECQuery = "psql -Uqradar -A -t -c \"select deployed_component.id,managedhost.hostname,managedhost.status from deployed_component join managedhost on deployed_component.managed_host_id=managedhost.id where component_id = (select id from component where type = 'eventcollector');\""

strECExistingCountQuery = "psql -Uqradar -A -t -c \"select count(*) from sensordevice where spconfig is NOT NULL;\""

# Query to find the protocol config with the given name
strfmtProtocolConfigIdQuery = "psql -Uqradar -A -t -c \"select id from sensorprotocolconfig where configname='%s';\""

# Query to find the device list belonging to specified config
strfmtSensorDeviceQuery = "psql -Uqradar -A -t -c \"select hostname from sensordevice where spconfig=%d and eccomponentid <> -1;\""

# Query to find the existing Windows devices list belonging to any config
strAllWindowsSensorDeviceQuery = "psql -Uqradar -A -t -c \"select hostname from sensordevice where spconfig in (select id from sensorprotocolconfig where spid=30) and eccomponentid <> -1;\""

# Query to find the number of devices belonging to specified config
strfmtSensorDeviceCountQuery = "psql -Uqradar -A -t -c \"select count(hostname) from sensordevice where spconfig=%d and eccomponentid <> -1;\""

# Sensor device insert statement
strfmtSensorDeviceInsert = "psql -Uqradar -A -t -c \"INSERT INTO sensordevice (id, hostname, devicename, devicedescription, devicecredibility, sourcecomponentid, eccomponentid, deviceenabled, autodiscovered, deployed, devicetypeid, spconfig, logonly, coalesce_events, store_event_payload, extension_id, extension_use_condition, parsing_order) VALUES (nextval('sensordevice_sequence'), '%s', '%s', '%s', 5, NULL, %d, true, false, false, 12, %d, false, true, true, NULL, 0, 1);\""

# Protocol config insert statement
strfmtProtocolConfigInsert = "psql -Uqradar -A -t -c \"INSERT INTO sensorprotocolconfig (id, spid, configname, defaultconfig) VALUES (nextval('sensorprotocolconfig_sequence'), 30, '%s', false);\"" 

# Protocol Config Param insert statement
strfmtProtocolConfigParameterInsert = "psql -Uqradar -A -t -c \"INSERT INTO sensorprotocolconfigparameters (name, sensorprotocolconfigid, value, paramtype) VALUES ('%s', %d, '%s', '%s');\""

# Query to list all deployed event collectors
strfmtSensorDeviceListQuery = "psql -Uqradar -A -t -c \"select deployed_component.id, deployed_component.name from deployed_component where component_id = (select id from component where type = 'eventcollector');\""

# Query to list all windows log source protocol configurations
strfmtWindowsLogSourceProtocolListQuery = "psql -Uqradar -A -t -c \"select id,configname from sensorprotocolconfig where spid = 30;\""

# Query to get next available protocol config id
strConfigIdMaxQuery = "psql -Uqradar -A -t -c \"select max(id) from sensorprotocolconfig where spid = 30;\""

# Query to retrieve sensor protocol config parameters to be copied
strfmtConfigParameterQuery = "psql -Uqradar -A -t -c \"select name, value, paramtype from sensorprotocolconfigparameters where sensorprotocolconfigid = %d;\""

# Command to check for paramtype column
strParamTypeCheck = "psql -Uqradar -A -t -c \"\d sensorprotocolconfigparameters\" | grep paramtype"

# select id,configname from sensorprotocolconfig where spid = 30;
################################################

class App(object):
	'''The main application class.'''
	def __init__(self):
		self.strInputFile = ''
		self.strConfigId = ''
		self.nConfigId = -1
		self.nEcId = -1
		self.bOneLogSourcePerConfigId = False
		
	def getLimit(self):
		limit = -1
		licenceFile = '/opt/qradar/conf/license'
		f = open(licenceFile, 'r');
		pat = re.compile(r'^\s*MAX_DEVICES\s*:\s*(\d+)\s*$')
		for line in f.readlines():
			#print 'REMOVE read; '+line
			m = pat.match(line)
			if m:
				limit = int(m.group(1))
				
				break
#		print 'getLimit REMOVE returning '+str(limit)	
		return limit
	
	def getExistingEcs(self):
		count = -1
		(stdin, stdout) = os.popen2(strECExistingCountQuery)
		for line in stdout.readlines():
			val = line.strip()
			#print 'REMOVE val='+val
			count = int(val)
#		print 'getExistingEcs '	+ str(count)
		return count
	
	def getAvailableECs(self):
		ecLimit = self.getLimit()
		hasEcLimit = ecLimit > -1
		if not hasEcLimit:
			return -1
		existingEcs = self.getExistingEcs()
		availableEcs = ecLimit - existingEcs
#		print 'getAvailable ECs '+str(availableEcs)
		return availableEcs
	
	def getLastConfigId(self):
		intval = -1
		(stdin, stdout) = os.popen2(strConfigIdMaxQuery)
		for line in stdout.readlines():
			val = line.strip()
			intval = int(val)
		return intval
	
	def Run(self):
		'''The main engine to run the application's logic.'''
		self.VerifyArguments()
		self.VerifyParamTypeColumnExists()

#		ecLimit = self.getLimit()
		#print 'ecLimit = '+str(ecLimit)
#		hasEcLimit = ecLimit > -1
		
		# Build currently assigned device list
		print 'Building current device list...'
		(stdin, stdout) = os.popen2(strfmtSensorDeviceQuery % (self.nConfigId))

		arrCurrentHostnames = []
		arrNewHostnames = []
		for line in stdout.readlines():
			arrCurrentHostnames.append(line.strip())
			
		print 'Found %d currently assigned devices.' % (len(arrCurrentHostnames))

		print 'Building ignore list...'
		(stdin, stdout) = os.popen2(strAllWindowsSensorDeviceQuery)

		arrIgnoreHostnames = []
		for line in stdout.readlines():
			arrIgnoreHostnames.append(line.strip())

		print 'Building new device list...'
		#This only leave uniques in the input file (removes duplicates with case insensitivity)
		strUniquesFile = "/usr/tmp/uniques"
		os.system('cat ' + self.strInputFile  + ' | sort | uniq -i > ' +  strUniquesFile)
		f = open(strUniquesFile, 'r');

		# Go through the file line by line. Strip off LFs and add to the result list anything we don't find in the current list.
		arrNewHostnames = [line.strip() for line in f.readlines() if len(line) > 1 and line.strip() not in arrIgnoreHostnames]

		f.close()

		constraint = ''
		availableEcs = self.getAvailableECs()
		hasEcLimit = availableEcs > -1
		 
		
		#print 'REMOVE existingEcs' + str(existingEcs)
		if hasEcLimit:
			constraint = ', however you can only add '+ str(availableEcs) + ' devices'
		print 'New device list contains '+str(len(arrNewHostnames))+' devices to be created from the specified file, '+self.strInputFile +constraint 
		
		intConfigId = -1
		arrConfigParams = []
		primaryIndex = 0;
		lines = []
				
		if self.bOneLogSourcePerConfigId:
			print 'Building config parameter list...'
			strInsertStmt = strfmtConfigParameterQuery % (self.nConfigId)
			(stdin, stdout) = os.popen2(strInsertStmt)
			for line in stdout.readlines():
				arrConfigParams.append([])
				lines = line.split("|")
				arrConfigParams[primaryIndex].append(lines[0].strip())
				arrConfigParams[primaryIndex].append(lines[1].strip())
				arrConfigParams[primaryIndex].append(lines[2].strip())
				primaryIndex += 1
				
		newCount = 0
		for strAddress in arrNewHostnames:
			#print 'REMOVE hasEcLimit='+str(hasEcLimit)+' availableEcs='+str(availableEcs)+' newCount='+str(newCount)
			if not hasEcLimit or hasEcLimit and newCount < availableEcs:
				if self.bOneLogSourcePerConfigId:
					strInsertStmt = strfmtProtocolConfigInsert % ('WindowsAuthServer@' + strAddress)
					(stdin, stdout) = os.popen2(strInsertStmt)
					sleep(1)
					self.nConfigId = self.getLastConfigId()
					self.strConfigId = str(self.nConfigId)
					print 'Inserting config parameters for config ' + self.strConfigId
					for param in arrConfigParams:
						strInsertStmt = strfmtProtocolConfigParameterInsert % (param[0], self.nConfigId, param[1], param[2])
						(stdin, stdout) = os.popen2(strInsertStmt) 
					
				print 'Inserting device ' + strAddress + ' into config "' + self.strConfigId + '" for event collector ' + str(self.nEcId)
				strInsertStmt = strfmtSensorDeviceInsert % (strAddress, 'WindowsAuthServer @ ' + strAddress, 'WindowsAuthServer device', self.nEcId, self.nConfigId)
				(stdin, stdout) = os.popen2(strInsertStmt)
				newCount = newCount + 1;
				
		
		print 'Rebuilding current device list, added '+str(newCount)+' devices...'

		# We need a sleep here because sometimes postgres doesn't like to give me the correct answer if I don't give it time to get its affairs in order.
		sleep(1)
		(stdin, stdout) = os.popen2(strfmtSensorDeviceCountQuery % (self.nConfigId))

		strCount = stdout.readline().strip()
		if self.bOneLogSourcePerConfigId == False:
			print 'There are now %d currently assigned devices for the "%s" protocol.' % (int(strCount), self.strConfigId)

	def SetInputFile(self, _strInputFile):
		'''The full path to the input file.'''
		if _strInputFile is None or len(_strInputFile) < 1:
			raise ValueError('Input filename must be specified')
		
		if not os.path.exists(_strInputFile):
			raise ValueError('The input file specified for the hostnames does not exist')

		self.strInputFile = _strInputFile

	def SetECId(self, _nEcId):
		'''The event collector id from which the logs will be gathered.'''
		self.nEcId = int(_nEcId)

		# Verify the ec id by doing a query from the database
		(stdin, stdout) = os.popen2(strECQuery)

		bValidEcId = False;
		arrValidEcIds = []
		for line in stdout.readlines():
			(ecsid, machine, status) = line.strip().split('|')
			if self.nEcId == int(ecsid):
				bValidEcId = True
				break
			else:
				arrValidEcIds.append('%s (%s - %s)' % (ecsid, machine, status))

		if not bValidEcId:
			raise ValueError('Looking at the database, the only valid ec ids available are:\n' + '\n'.join(arrValidEcIds))

	def SetProtocolConfigId(self, _strConfigId):
		'''The full name of the protocol config that the imported devices will belong to.'''
		if _strConfigId is None or len(_strConfigId) < 1:
			raise ValueError('Protocol config id must be specified')
		self.strConfigId = _strConfigId

#		# Get the target proto config id (There should be zero or one line, empty or not)
#		(stdin, stdout) = os.popen2(strfmtProtocolConfigIdQuery % (self.strConfigName))
#		strProtocolId = stdout.readline()
#		if strProtocolId is None or len(strProtocolId.strip()) == 0:
#			raise ValueError('A sensor protocol with the specified name does not exist.')
		self.nConfigId = int(_strConfigId.strip())
		
	def SetOneLogSourcePer(self, _bOneLogSourcePerConfigId):
		self.bOneLogSourcePerConfigId = _bOneLogSourcePerConfigId

	def VerifyArguments(self):
		'''One last quick check to make sure are args are sane.'''
		if self.strInputFile is None or len(self.strInputFile) < 1:
			raise ValueError('Input filename must be specified')
		if self.strConfigId is None or len(self.strConfigId) < 1:
			raise ValueError('Protocol config name must be specified')
		if self.nEcId < 0:
			raise ValueError('EC id must be specified and exist')
		if self.nConfigId < 0:
			raise ValueError('Sensor protocol must be specified and exist')
		
	def VerifyParamTypeColumnExists(self):
		(stdin, stdout) = os.popen2(strParamTypeCheck)
		strResult = stdout.readline().strip()
		if strResult == '':
			print 'Error: the QRadar system must be version 6.3.1, or 6.3.0 with a patch level of at least 135860 in order to proceed.'
			sys.exit(-1)

	def showCollectors(self):
		strfmtSensorDeviceListQuery
#       strfmtWindowsLogSourceProtocolListQuery
		(stdin, stdout) = os.popen2(strfmtSensorDeviceListQuery)
		lines = stdout.readlines()
		lineCount = len(lines)
		print 'Showing '+str(lineCount)+' Deployed Event Collectors; (use id column in \'-e [id]\''
		for line in lines:
			print '\tid|name'
			print '\t======='
			if lineCount > 0 :
				print '\t'+line
			print ''
		
		(stdin, stdout) = os.popen2(strfmtWindowsLogSourceProtocolListQuery)
		lines = stdout.readlines()
		lineCount = len(lines)
		print 'Showing '+str(lineCount)+' Windows Log Source Protocols (use id column in \'-p [id]\''
		if len(lines) > 0 :
			print '\tid|name'
			print '\t======='
			for line in lines:
				print '\t'+line
			print ''
			
		#thought it would be good to show how many devices can be inserted too
		availableEcs = int(self.getAvailableECs())
		hasLimit = availableEcs > -1
		if hasLimit:
			print 'This '+self.getSystemVersion()+' installation can handle '+str(availableEcs) +' more event collectors.'
		else:
			print 'This '+self.getSystemVersion()+' installation has no limit to the number of event collectors.'	
			
	def getSystemVersion(self):
		#cat /opt/qradar/conf/nva.conf|grep BUILD_VERSION
		
		nvaFile = '/opt/qradar/conf/nva.conf'
		f = open(nvaFile, 'r');
		pat = re.compile(r'^\s*BUILD_VERSION\s*=\s*(\d_\d)\w+')
		for line in f.readlines():
#			print 'checking '+line
			m = pat.match(line)
			if m:
				limit = m.group(1)
				f.close()
				return limit
#			else:
#				print 'nf in line '+ line
		#print 'REMOVE returning '+str(limit)
		f.close()	
		return ''			
		
################################################

def __Run():
	'''Engine function called from main. Runs everything.'''
	parser = OptionParser(usage='usage: %prog -i <input_file> -p <protocol config id> -e <ec_id> \n Use -o flag if only one log source per protocol config id is desired \n Run %prog -d to get listing of appropriate values', version = '%prog ' + scriptVersion)

	# Command-line options
	parser.add_option('-i', '--input_file', action='store', dest='strInputFile', default=r'',metavar='INPUT_FILE',help='The full path to the input file containing the machine hostnames/addresses to import.')
	parser.add_option('-p', '--protocol_config_id', action='store', metavar='PROTOCOL_CONFIG_ID', dest='strProtocolConfigID', help='<Windows Event Log Protocol configuration Id> Specify the Id of the protocol to which these devices will belong.')
	parser.add_option('-e', '--ec_id', action='store', dest='nEcId', metavar='EVENT_COLLECTOR_ID', help='An existing event collector id from which the logs will be gathered.')
	parser.add_option('-d', '--display', action='store_true', dest='display', default=False, metavar='DISPLAY', help='Optional: Standalone. to display lists of deployed event collectors and protocols.')
	parser.add_option('-o', '--one_logsource_per_config', action='store_true', dest='oneLogSourcePerConfigId', default=False, metavar='ONE_LOGSOURCE_PER_CONFIG', help='Optional. Use to ensure each log source gets its own protocol config id, each will have the same config parameters as the existing config specified with -p. A Deploy will be required after using this option. ')
	 
	(options, args) = parser.parse_args()
	
	
	app = App()
	if options.display:
		app.showCollectors()
	else: 	
		# Configuration of the app object
		try:
			app.SetInputFile(options.strInputFile)
			app.SetProtocolConfigId(options.strProtocolConfigID)
			app.SetECId(options.nEcId)
			if options.oneLogSourcePerConfigId:
				app.SetOneLogSourcePer(options.oneLogSourcePerConfigId)
		except ValueError, ve:
			print '\nThere was a problem with one of the command-line arguments: \n' + str(ve) + '\n'
			parser.print_help()
			sys.exit(-1)
		except Exception, e:
			parser.print_usage()
			sys.exit(-1)
	
		try:
			print 'Running...'
			app.Run()
		except KeyboardInterrupt:
			print
			print 'Run aborted by user.'
			print
		except SystemExit:
			print
		except Exception, e:
			print '\nSomething weird and awful happened: ' + str(e)

	print 'Finished.'

# If this is run as the main module (i.e. not imported), run the __Run() driver function.
if __name__ == "__main__":
	__Run()	

